"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const build_step_1 = require("./build-step");
/**
 * Runs files through a series of plugins.
 * @internal
 */
async function runBuild(files, steps, concurrency, context) {
    let promises = [], promise;
    let summary = {
        input: { fileCount: 0, fileSize: 0 },
        output: { fileCount: 0, fileSize: 0 },
        time: { start: new Date(), end: new Date(), elapsed: 0 },
    };
    // Remove the contents from the changed files so the build context is lightweight
    // for cloning across the thread boundary
    context.changedFiles = context.changedFiles.map(lightweightChangedFile);
    // Collect metrics on the input files
    let input = updateBuildSummary(summary, "input", files);
    // Chain the build steps together, with each one accepting the output of the previous one as input
    for (let step of steps) {
        let output = new utils_1.IterableWriter();
        promise = build_step_1.runBuildStep(step, concurrency, input, output, context);
        promises.push(promise);
        input = output.iterable;
    }
    // Collect metrics on the final output files
    let finalOutput = updateBuildSummary(summary, "output", input);
    // Wait for all build steps to finish
    promises.push(utils_1.drainIterable(finalOutput));
    await Promise.all(promises);
    // Update the build summary
    summary.time.end = new Date();
    summary.time.elapsed = summary.time.end.getTime() - summary.time.start.getTime();
    return summary;
}
exports.runBuild = runBuild;
/**
 * Creates a "lightweight" copy of a `ChangedFile` object without its contents.
 */
function lightweightChangedFile(changedFile) {
    let copy = utils_1.createChangedFile({ ...changedFile });
    copy.contents = Buffer.alloc(0);
    return copy;
}
/**
 * Updates the `input` or `output` metrics of the given `BuildSummary`.
 */
function updateBuildSummary(summary, io, files) {
    let generator = gatherFileMetrics(summary, io, files);
    generator.all = utils_1.iterateAll;
    return generator;
}
// tslint:disable-next-line: no-async-without-await
async function* gatherFileMetrics(summary, io, files) {
    for await (let file of files) {
        summary[io].fileCount++;
        summary[io].fileSize += file.size;
        yield file;
    }
}
//# sourceMappingURL=build.js.map